@@ -12,6 +12,7 @@ import os
 import re
 import time
 from contextlib import suppress
+from importlib.util import find_spec
 from pathlib import Path
 from typing import Any, Dict, Iterable, Tuple
 
@@ -23,7 +24,6 @@ from django.db.models.functions import Coalesce
 
 from resolwe.flow.executors import constants
 from resolwe.flow.executors.prepare import BaseFlowExecutorPreparer
-from resolwe.flow.executors.protocol import ExecutorFiles
 from resolwe.flow.models import Data, DataDependency, Process
 from resolwe.storage import settings as storage_settings
 from resolwe.storage.connectors import connectors
@@ -58,9 +58,6 @@ class Connector(BaseConnector):
         self.kubernetes_namespace = getattr(settings, "KUBERNETES_SETTINGS", {}).get(
             "namespace", "default"
         )
-        self.runtime_dir = Path(
-            storage_settings.FLOW_VOLUMES["runtime"]["config"]["path"]
-        )
 
     def _get_upload_dir(self) -> str:
         """Get upload path.
@@ -177,29 +174,32 @@ class Connector(BaseConnector):
         This is done using configmap which is mostly static. This command
         returns description for this configmap.
         """
-        socket_utils_path: Path = (
-            self.runtime_dir / "executors" / ExecutorFiles.SOCKET_UTILS
-        )
-        processing_startup_path: Path = (
-            self.runtime_dir / "executors" / ExecutorFiles.STARTUP_PROCESSING_SCRIPT
-        )
-        constants_path: Path = self.runtime_dir / "executors" / ExecutorFiles.CONSTANTS
-
-        startup_content = processing_startup_path.read_text()
-        socket_utils_content = socket_utils_path.read_text()
-        constants_content = constants_path.read_text()
         passwd_content = "root:x:0:0:root:/root:/bin/bash\n"
         passwd_content += f"user:x:{os.getuid()}:{os.getgid()}:user:{os.fspath(constants.PROCESSING_VOLUME)}:/bin/bash\n"
         group_content = "root:x:0:\n"
         group_content += f"user:x:{os.getgid()}:user\n"
 
-        data = {
-            "passwd": passwd_content,
-            "group": group_content,
-            "startup-script": startup_content,
-            "socket-utils": socket_utils_content,
-            "constants": constants_content,
+        data = dict()
+        modules = {
+            "communicator": "resolwe.process.communicator",
+            "bootstrap-python-runtime": "resolwe.process.bootstrap_python_runtime",
+            "socket-utils": "resolwe.flow.executors.socket_utils",
+            "constants": "resolwe.flow.executors.constants",
+            "startup-script": "resolwe.flow.executors.startup_processing_container",
         }
+        for module, full_module_name in modules.items():
+            spec = find_spec(full_module_name)
+            assert (
+                spec is not None and spec.origin is not None
+            ), f"Unable to determine module {full_module_name} source code."
+            data[module] = Path(spec.origin).read_text()
+
+        data.update(
+            {
+                "passwd": passwd_content,
+                "group": group_content,
+            }
+        )
 
         data_md5 = hashlib.md5(json.dumps(data, sort_keys=True).encode()).hexdigest()
         configmap_name = f"configmap-files-{data_md5}"
@@ -237,6 +237,8 @@ class Connector(BaseConnector):
                     "name": volume_name,
                     "hostPath": {"path": os.fspath(volume_config["config"]["path"])},
                 }
+            elif volume_config["type"] == "temporary_directory":
+                return {"name": volume_name, "emptyDir": {}}
             else:
                 raise RuntimeError(
                     f"Unsupported volume configuration: {volume_config}."
@@ -250,9 +252,7 @@ class Connector(BaseConnector):
             {
                 "name": "files-volume",
                 "configMap": {"name": files_configmap_name},
-            },
-            {"name": "sockets-volume", "emptyDir": {}},
-            {"name": "secrets-volume", "emptyDir": {}},
+            }
         ]
         tools_configmaps = self._get_tools_configmaps(core_api)
         for index, configmap_name in enumerate(tools_configmaps):
@@ -290,7 +290,7 @@ class Connector(BaseConnector):
                 "readOnly": False,
             },
             {
-                "name": "secrets-volume",
+                "name": constants.SECRETS_VOLUME_NAME,
                 "mountPath": os.fspath(constants.SECRETS_VOLUME),
                 "readOnly": False,
             },
@@ -320,12 +320,12 @@ class Connector(BaseConnector):
         """
         mount_points = [
             {
-                "name": "sockets-volume",
+                "name": constants.SOCKETS_VOLUME_NAME,
                 "mountPath": os.fspath(constants.SOCKETS_VOLUME),
                 "readOnly": False,
             },
             {
-                "name": "secrets-volume",
+                "name": constants.SECRETS_VOLUME_NAME,
                 "mountPath": os.fspath(constants.SECRETS_VOLUME),
                 "readOnly": False,
             },
@@ -401,11 +401,21 @@ class Connector(BaseConnector):
                 "subPath": "constants",
             },
             {
-                "name": "sockets-volume",
+                "name": "files-volume",
+                "mountPath": f"/{constants.BOOTSTRAP_PYTHON_RUNTIME}",
+                "subPath": "bootstrap-python-runtime",
+            },
+            {
+                "name": "files-volume",
+                "mountPath": "/communicator.py",
+                "subPath": "communicator",
+            },
+            {
+                "name": constants.SOCKETS_VOLUME_NAME,
                 "mountPath": os.fspath(constants.SOCKETS_VOLUME),
             },
             {
-                "name": "secrets-volume",
+                "name": constants.SECRETS_VOLUME_NAME,
                 "mountPath": os.fspath(constants.SECRETS_VOLUME),
                 "readOnly": True,
             },
@@ -417,25 +427,6 @@ class Connector(BaseConnector):
                     "mountPath": f"/usr/local/bin/resolwe/{tool_index}",
                 }
             )
-
-        from resolwe.flow.managers import manager
-
-        execution_engine = manager.get_execution_engine(execution_engine_name)
-        runtime_volumes = execution_engine.prepare_volumes()
-        subpath = storage_settings.FLOW_VOLUMES["runtime"]["config"].get("subpath", "")
-        if subpath:
-            runtime_volumes = {
-                f"{subpath}/{src}": dst for src, dst in runtime_volumes.items()
-            }
-
-        mount_points += [
-            {
-                "name": "runtime",
-                "mountPath": dst,
-                "subPath": src,
-            }
-            for src, dst in runtime_volumes.items()
-        ]
         return mount_points
 
     def _persistent_ebs_claim(
@@ -495,6 +486,17 @@ class Connector(BaseConnector):
                 sanitized_label = sanitized_label[:max_length].strip("-_.")
         return sanitized_label
 
+    def _image_mapper(self, image_name: str, mapper: Dict[str, str]) -> str:
+        """Transform the image name if necessary.
+
+        When image_name starts with one of the keys defined in the mapper the
+        matching part is replaced with the corresponding value.
+        """
+        for key in mapper:
+            if image_name.startswith(key):
+                return f"{mapper[key]}{image_name[len(key):]}"
+        return image_name
+
     def start(self, data: Data, listener_connection: Tuple[str, str, str]):
         """Start process execution.
 
@@ -578,11 +580,13 @@ class Connector(BaseConnector):
             # mount my /tmp directory to the /seccomp directory in minikube.
             annotations["seccomp.security.alpha.kubernetes.io/pod"] = "runtime/default"
 
+        mapper = getattr(settings, "FLOW_CONTAINER_IMAGE_MAP", {})
         communicator_image = getattr(
             settings,
             "FLOW_DOCKER_COMMUNICATOR_IMAGE",
             "public.ecr.aws/s4q6j6e8/resolwe/com:latest",
         )
+        communicator_image = self._image_mapper(communicator_image, mapper)
 
         requirements = data.process.requirements.get("executor", {}).get("docker", {})
         processing_container_image = str(
@@ -595,6 +599,9 @@ class Connector(BaseConnector):
                 ),
             ),
         )
+        processing_container_image = self._image_mapper(
+            processing_container_image, mapper
+        )
 
         job_type = dict(Process.SCHEDULING_CLASS_CHOICES)[data.process.scheduling_class]
         job_description = {
@@ -734,7 +741,7 @@ class Connector(BaseConnector):
         """
         return "{}-{}".format(prefix, data_id)
 
-    def submit(self, data: Data, runtime_dir: str, argv: Any):
+    def submit(self, data: Data, argv):
         """Run process.
 
         For details, see
