@@ -16,13 +16,14 @@ logger = logging.getLogger(__name__)
 class AwsS3Connector(BaseStorageConnector):
     """Amazon S3 storage connector."""
 
-    REQUIRED_SETTINGS = ["bucket", "credentials"]
+    REQUIRED_SETTINGS = ["bucket"]
 
     def __init__(self, config: dict, name: str):
         """Connector initialization."""
         super().__init__(config, name)
         self.bucket_name = config["bucket"]
         self.supported_hash = ["awss3etag"]
+        self.refresh_hash_after_transfer = ["awss3etag"]
         self.hash_propery = {"awss3etag": "e_tag"}
         self.multipart_threshold = self.config.get(
             "multipart_threshold", self.CHUNK_SIZE
@@ -32,6 +33,13 @@ class AwsS3Connector(BaseStorageConnector):
         )
         self.use_threads = True
 
+        # Ensured by TLS protocol used for transport.
+        self.get_ensures_data_integrity = True
+        # Boto library computes MD5 checksums for every chunk of data it
+        # transmits to the bucket and compares it to the actual md5 of the
+        # data received by the server.
+        self.put_ensures_data_integrity = True
+
         self._session = None
         self._client = None
         self._sts = None
@@ -39,17 +47,22 @@ class AwsS3Connector(BaseStorageConnector):
 
     def _initialize(self):
         """Initializaton."""
-        credentials = self.config["credentials"]
-        with open(credentials) as f:
-            settings = json.load(f)
-        self.session = boto3.Session(
-            aws_access_key_id=settings["AccessKeyId"],
-            aws_secret_access_key=settings["SecretAccessKey"],
-            aws_session_token=settings.get("SessionToken"),
-            region_name=self.config.get("region_name"),
-        )
+        session_kwargs = {"region_name": self.config.get("region_name")}
+        # Credentials may be given explicitely in config. When they are not
+        # the system credentials are used instead.
+        if "credentials" in self.config:
+            with open(self.config["credentials"]) as f:
+                settings = json.load(f)
+                session_kwargs["aws_access_key_id"] = settings["AccessKeyId"]
+                session_kwargs["aws_secret_access_key"] = settings["SecretAccessKey"]
+
+        self.session = boto3.Session(**session_kwargs)
+
         self.awss3 = self.session.resource("s3")
-        self.client = self.session.client("s3")
+
+        self.client = self.session.client(
+            "s3", config=botocore.client.Config(signature_version="s3v4")
+        )
         self.sts = self.session.client("sts")
 
     def __getattr__(self, name):
@@ -76,6 +89,48 @@ class AwsS3Connector(BaseStorageConnector):
             ExtraArgs=extra_args,
         )
 
+    def multipart_push_start(self, url, size=None):
+        """Start a multipart upload.
+
+        :returns: the upload id.
+        """
+        url = os.fspath(url)
+        mime_type = mimetypes.guess_type(url)[0]
+        upload_args = {"Bucket": self.bucket_name, "Key": url}
+        if mime_type is not None:
+            upload_args["ContentType"] = mime_type
+        response = self.client.create_multipart_upload(**upload_args)
+        return response["UploadId"]
+
+    def multipart_push(self, upload_id, url, part_number, chunk_size, data, md5=None):
+        """Upload a single part of multipart upload."""
+        result = self.client.upload_part(
+            Body=data,
+            Bucket=self.bucket_name,
+            ContentMD5=md5,
+            Key=os.fspath(url),
+            PartNumber=part_number,
+            UploadId=upload_id,
+        )
+        return {"ETag": result["ETag"], "PartNumber": part_number}
+
+    def multipart_push_complete(self, upload_id, url, completed_chunks):
+        """Complete the multipart push."""
+        return self.client.complete_multipart_upload(
+            Bucket=self.bucket_name,
+            Key=os.fspath(url),
+            UploadId=upload_id,
+            MultipartUpload={"Parts": completed_chunks},
+        )
+
+    def multipart_push_abort(self, upload_id, url):
+        """Abort multiport upload."""
+        return self.cliest.abort_multipart_upload(
+            Bucket=self.bucket_name,
+            Key=os.fspath(url),
+            UploadId=upload_id,
+        )
+
     @validate_urls
     @validate_url
     def delete(self, url, urls):
